@@ -11,6 +11,7 @@ fmt_model = {'color': 'blue', 'linestyle': 'solid', 'linewidth': 1.2, 'marker':
 fmt_model_truss = {'color': 'green', 'linestyle': 'solid', 'linewidth': 1.2, 'marker': 'o', 'markersize': 6, 'markerfacecolor': 'white'}
 fmt_model_nodes_only = {'color': 'blue', 'linestyle': 'solid', 'linewidth': 1.2, 'marker': '.', 'markersize': 6}
 fmt_model_loads = {'color': 'black', 'linestyle': 'solid', 'linewidth': 1.2, 'marker': '', 'markersize': 1}
+fmt_model_rigid_offset = {'color': 'black', 'linestyle': 'solid', 'linewidth': 3.2, 'marker': '.', 'markersize': 1}
 
 fmt_model_joint2d = {'color': 'black', 'linestyle': 'solid', 'linewidth': 1.2, 'marker': '', 'markersize': 1}
 fmt_model_secforce = {'color': 'black', 'linestyle': 'solid', 'linewidth': 1.2, 'marker': '', 'markersize': 1}
@@ -66,6 +67,7 @@ class EleClassTag:
     ForceBeamColumn2d = 73
     ForceBeamColumn3d = 74
     TimoshenkoBeamColumn2d = 63
+    TimoshenkoBeamColumn3d = 631
     ElasticTimoshenkoBeam2d = 145
     ElasticTimoshenkoBeam3d = 146
     tri3n = 33
@@ -86,6 +88,10 @@ class EleClassTag:
     ASDShellT3 = 204
     ShellMITC4 = 53
     ShellMITC9 = 54
+    ShellDKGQ = 156
+    ShellNLDKGQ = 157
+    ShellDKGT = 167
+    ShellNLDKGT = 168        
     Joint2D = 82
     Joint3D = 83
     MVLEM = 162
@@ -93,6 +99,8 @@ class EleClassTag:
     MVLEM_3D = 212
     SFI_MVLEM_3D = 213
     TwoNodeLink = 86
+    Pipe = 269
+    CurvedPipe = 270
 
 
 class LoadTag:
